resource "aws_lb" "default" {
  load_balancer_type = "application"
  name               = "${var.name}-${var.env}-alb"
  subnets            = var.jz_vpc.subnets.*.id

  security_groups = [
    aws_security_group.alb.id,
  ]

  enable_deletion_protection = true

  tags = {
    Name = "${var.name}-${var.env}-alb"
    Env  = var.env
  }
}

locals {
  # Mapping of ALB listeners from config port_mapping.
  listeners = {
    for i, z in var.port_mapping:
    z.alb_port => {
      app_port   = z.app_port
      app_schema = upper(z.app_schema)
      alb_port   = z.alb_port
      alb_schema = upper(z.alb_schema)
      is_secure  = upper(z.alb_schema) == "HTTPS" || upper(z.alb_schema) == "SSL"
      host       = lookup(z, "host", "")
    }...
  }
  # Mapping of host-based-routed 443 (HTTPS) records.
  https_with_hosts = {
    for i,z in lookup(local.listeners, "443", []):
    i => {
      app_port = z.app_port
      host = z.host
    }
    if lookup(z, "host", "") != ""
  }
  # Mapping of ALB target groups from config port_mapping.
  target_groups = {
    for i, z in var.port_mapping:
    z.app_port => {
      app_port   = z.app_port
      app_schema = upper(z.app_schema)
    }...
  }
  # TGs * instances
  instance_ids_flatten = flatten([
    var.jz_swarm_project.instance_ids.leader,
    var.jz_swarm_project.instance_ids.managers.*,
    var.jz_swarm_project.instance_ids.workers.*]
  )
  target_group_arns    = values(aws_lb_target_group.default).*.arn
  tg_x_instance        = setproduct(local.target_group_arns, local.instance_ids_flatten)
}

resource "aws_lb_listener" "default" {
  for_each = local.listeners

  load_balancer_arn = aws_lb.default.arn
  port              = each.value[0].alb_port
  protocol          = each.value[0].alb_schema
  ssl_policy        = each.value[0].is_secure ? "ELBSecurityPolicy-2016-08" : null
  certificate_arn   = each.value[0].is_secure ? data.aws_acm_certificate.default[0].arn : null

  dynamic "default_action" {
    for_each = length(compact([for i,z in each.value: i if z.host != ""])) == 0 ? [1] : []
    content {
      type             = "forward"
      target_group_arn = aws_lb_target_group.default[each.value[0].app_port].arn
    }
  }

  dynamic "default_action" {
    for_each = length(compact([for i,z in each.value: i if z.host != ""])) > 0 ? [1] : []
    content {
      type             = "fixed-response"
      fixed_response {
          content_type = "text/plain"
          status_code  = "404"
          message_body = "Not Found"
      }
    }
  }
}

resource "aws_lb_listener" "redirect80to443" {
  count = var.redirect_80_to_443 == true ? 1 : 0

  load_balancer_arn = aws_lb.default.arn
  port              = 80
  protocol          = "HTTP"

  default_action {
    type = "redirect"
    redirect {
      port        = "443"
      protocol    = "HTTPS"
      status_code = "HTTP_301"
    }
  }
}

resource "aws_lb_listener_rule" "https" {
  for_each = local.https_with_hosts

  listener_arn = aws_lb_listener.default["443"].arn

  action {
    type             = "forward"
    target_group_arn = aws_lb_target_group.default[each.value.app_port].arn
  }

  condition {
    field  = "host-header"
    values = [
      each.value.host
    ]
  }
}

resource "aws_lb_target_group" "default" {
  for_each = local.target_groups

  name     = "${var.name}-${var.env}-${each.key}-tg"
  vpc_id   = var.jz_vpc.self.id

  port     = each.value[0].app_port
  protocol = each.value[0].app_schema

  health_check {
      enabled             = true
      healthy_threshold   = 2
      unhealthy_threshold = 5
      interval            = 30
      timeout             = 5
      matcher             = "200-499"
  }
}

resource "aws_lb_target_group_attachment" "default" {
  count = length(local.tg_x_instance)

  target_group_arn = local.tg_x_instance[count.index][0]
  target_id        = local.tg_x_instance[count.index][1]
}

data "aws_acm_certificate" "default" {
  count       = var.ssl_cert_domain != "" ? 1 : 0
  domain      = var.ssl_cert_domain
  most_recent = true
}
